@@ -10,6 +10,27 @@ from Crypto.Cipher import Blowfish
 from Crypto.Protocol.KDF import PBKDF2
 from cryptography.fernet import Fernet
 from random import randbytes, shuffle
+from platform import system as get_platform
+
+class NoKeyError(Exception):
+    """
+    Raised when no key was provided to a cipher object.
+    """
+    pass
+
+
+class BadKeyError(Exception):
+    """
+    Raised when the key given does not match the encryption key signature.
+    """
+    pass
+
+
+class InvalidCipherArgument(Exception):
+    """
+    Raised when a parameter for a cipher is not provided.
+    """
+    pass
 
 
 def hash_(StringToHash=str, hash_code = str, return_type = "hex") -> str:
@@ -151,7 +172,7 @@ class Ciphers:
     # misc methods
 
 
-    def generate_symmetric_key(self, salt) -> 'tuple[bytes, bytes]':
+    def generate_symmetric_key(self, salt=None) -> 'tuple[bytes, bytes]':
         "Method for generating keys. Will return tuple (key, salt) if salt is not provided."
         if salt is None:
             salt_ = os.urandom(32)
@@ -187,10 +208,11 @@ class Ciphers:
         # checking if file is hidden
         if "/." in path:
             return 4
-        # checking if file is part of the root filesystem and not a part of ChromeOS for linux in Chromebooks
-        if "/home/" not in path and "/mnt/chromeos/MyFiles" not in path:
-            return 5
-        # checking if decrypt is cancelleD
+        # checking if file is part of the Linux root filesystem
+        if get_platform() == 'Linux':
+            if "/home/" not in path:
+                return 5
+        # checking if decrypt is cancelled
         try:
             if os.path.isfile(path) is True:
                 with open (path, "rb") as file:
@@ -250,9 +272,10 @@ class Ciphers:
 
     def AES_encrypt_file(self, path=str) -> int:
         """
-        A method for encrypting a file/folder object.
+        A method for encrypting a file/folder object with AES.
 
-        Takes only a file path as an argument.
+        Parameters:
+            `path`: The path of the file/folder to be encrypted.
 
         Return codes:
             0: File encrypt successful.
@@ -260,7 +283,7 @@ class Ciphers:
             2: Path is invalid.
             3: App does not have permissions required to access file.
             4: File is hidden.
-            5: File is part of root filesystem and not part of ChromeOS.
+            5: File is part of root filesystem (Linux only)
             6: File encryption marker detected during encryption.
         """
         # encryption for single file
@@ -321,7 +344,7 @@ class Ciphers:
 
     def AES_decrypt_file(self, path=str) -> int:
         """
-        A method for decrypting an encrypted file.
+        A method for decrypting an encrypted file that was encrypted with AES.
 
         Takes only a file path as an argument.
 
@@ -354,7 +377,7 @@ class Ciphers:
             key = self.generate_symmetric_key(salt=salt)
             key_hash = hash.sha256(key); key_signature = key_hash.digest()
             if file_signature != key_signature:
-                return 6
+                raise BadKeyError("Keys are not the same.")
             # setting cipher and decrypting the data
             cipher = AES.new(key, AES.MODE_CFB, iv=iv)
             original_data = cipher.decrypt(ciphered_data)
@@ -393,12 +416,39 @@ class Ciphers:
             return 0
         
 
-    def AES_encrypt_string():
-        pass
+    def AES_encrypt_string(self, key=bytes or None, string_to_encrypt=bytes):
+        """
+        Method for encrypting strings using the AES encryption alogrithm in CFB mode.
+
+        Parameters:
+            `string_to_encrypt`: the string being encrypted, in bytes
+            `key`: the key to encrypt the string with. If not provided, will generate a new key using the `Ciphers` class `password` attribute.
+        returns a tuple of `(ciphered_string, iv, key)`
+        """
+        if key is None:
+            salt = os.urandom(32)
+            key = self.generate_symmetric_key(salt)
+        iv = os.urandom(16)
+        cipher = Blowfish.new(key, Blowfish.MODE_CFB, iv=iv)
+        ciphered_string = cipher.encrypt(string_to_encrypt)
+        return (ciphered_string, iv, key)
 
 
-    def AES_decrypt_string():
-        pass
+    def AES_decrypt_string(self, encrypted_string:bytes, key:bytes, iv:bytes):
+        """
+        Method for encrypting strings using the AES encryption alogrithm in CFB mode.
+
+        Parameters:
+            `encrypted_string`: The string to be decrypted, in bytes.
+            `key`: The key used to encrypt the string.
+            `iv`: The Intialization vector used for encryption.
+        returns the decrypted string in bytes format.
+        """
+        if key is None:
+            raise NoKeyError("No key was provided for the cipher.")
+        cipher = Blowfish.new(key, Blowfish.MODE_CFB, iv=iv)
+        decrypted_string = cipher.decrypt(encrypted_string)
+        return (decrypted_string)
 
 
     # Blowfish methods
@@ -406,7 +456,19 @@ class Ciphers:
 
     def BLO_encrypt_file(self, path=str):
         """
-        A method for encrypting a file using the Blowfish encryption algorithm.
+        A method for encrypting a file/folder object with Blowfish in CFB mode..
+
+        Parameters:
+            `path`: The path of the file/folder to be encrypted.
+
+        Return codes:
+            0: File encrypt successful.
+            1: Path is None.
+            2: Path is invalid.
+            3: App does not have permissions required to access file.
+            4: File is hidden.
+            5: File is part of root filesystem (Linux only)
+            6: File encryption marker detected during encryption.
         """
         # file encryption
         if os.path.isfile(path) is True:
@@ -467,7 +529,21 @@ class Ciphers:
 
     def BLO_decrypt_file(self, path=str):
         """
-        method for decrypting a file encrypted with the Blowfish encryption alogrithm
+        A method for decrypting an encrypted file or folder that was encrypted using Blowfish.
+
+        Parameters:
+            `path`: The path of the file as a string.
+
+        return codes:
+            0: File decrypt successful.
+            1: Path is None.
+            2: File path is invalid.
+            3: App does not have permissions required to access file.
+            4: File is hidden.
+            5: File is part of root filesystem (linux Debian based distros only).
+            6: File was encrypted with a different key.
+            7: File was not encrypted.
+            8: File was encrypted using a different alogorithm.
         """
         # file decryption
         if os.path.isfile(path) is True:
@@ -487,7 +563,7 @@ class Ciphers:
             key = self.generate_symmetric_key(salt=salt)
             key_hash = hash.sha256(key); key_signature = key_hash.hexdigest()
             if key_signature != file_signature:
-                return 6
+                raise BadKeyError("Key provided was different from the one used for encryption.")
             # setting cipher and decrypting data
             cipher = Blowfish.new(key, Blowfish.MODE_CFB, iv=iv)
             # writing
@@ -526,39 +602,54 @@ class Ciphers:
                         file.write(original_data)
 
 
-    def BLO_encrypt_string(self, key=bytes or None, string_=str):
+    def BLO_encrypt_string(self, string_to_encrypt:bytes, key=None):
         """
-        Method for encrypting strings using the Blowfish encryption alogrithm.
-        returns a tuple of (ciphered_string, iv, salt)
+        Method for encrypting strings using the Blowfish encryption alogrithm in CFB mode.
+
+        Parameters:
+            `string_to_encrypt`: the string being encrypted, in bytes
+            `key`: the key to encrypt the string with. If not provided, will generate a new key using the `Ciphers` class `password` attribute.
+        returns a tuple of `(ciphered_string, iv, key)`
         """
         if key is None:
             salt = os.urandom(32)
             key = self.generate_symmetric_key(salt)
         iv = os.urandom(8)
         cipher = Blowfish.new(key, Blowfish.MODE_CFB, iv=iv)
-        ciphered_string = cipher.encrypt(string_)
+        ciphered_string = cipher.encrypt(string_to_encrypt)
         return (ciphered_string, iv, key)
 
 
-    def BLO_decrypt_string(self, encrypted_string=str, key=bytes, iv=bytes):
+    def BLO_decrypt_string(self, encrypted_string:bytes, key:bytes, iv:bytes):
         """
-        Method for encrypting strings using the Blowfish encryption alogrithm.
-        returns a tuple of (ciphered_string, iv, salt)
+        Method for decrypting strings using the Blowfish encryption alogrithm in CFB mode.
+
+        Parameters:
+            `encrypted_string`: The string to be decrypted, in bytes.
+            `key`: The key used to encrypt the string.
+            `iv`: The Intialization vector used for encryption.
+        returns the decrypted string in bytes format.
         """
         if key is None:
-            return 1
+            raise NoKeyError("No key was provided for the cipher.")
+        if iv is None:
+            raise InvalidCipherArgument("No intialization vector was provided for the cipher.")
         cipher = Blowfish.new(key, Blowfish.MODE_CFB, iv=iv)
         decrypted_string = cipher.decrypt(encrypted_string)
         return (decrypted_string)
 
+
     # FerNet methods
 
 
-    def FerNet_encrypt_file(self, path=str, key = str or None):
+    def FerNet_encrypt_file(self, path:str, key=None):
+        # file encryption
         if os.path.isfile(path) is True:
+            #chyecking if file is suitable
             return_code = self.check_path_validity(path, decrypting=False, type_="FER")
             if return_code != 0:
                 return return_code
+            # generating key if no key is provided
             if key is None:
                 key = self.generate_FerNet_key()
             f = Fernet(key)
@@ -588,49 +679,188 @@ class Ciphers:
             return 0
     
 
-    def FerNet_decrypt_file(self, path=str, key=bytes):
+    def FerNet_decrypt_file(self, path=str, key=None):
+        """
+        A method for decrypting an encrypted file or folder that was encrypted using FerNet.
+
+        Parameters:
+            `path`: The path of the file as a string.
+
+            `key`: The key to encrypt the file with. Will raise a `NoKeyError` if not provided.
+
+        return codes:
+            0: File decrypt successful.
+            1: Path is None.
+            2: File path is invalid.
+            3: App does not have permissions required to access file.
+            4: File is hidden.
+            5: File is part of root filesystem (linux Debian based distros only).
+            6: File was encrypted with a different key.
+            7: File was not encrypted.
+            8: File was encrypted using a different alogorithm.
+        """
+        if key is None:
+            raise NoKeyError("No key was provided for decryption")
+        # file decryption
         if os.path.isfile(path) is True:
             return_code = self.check_path_validity(path, decrypting=True, type_="FER")
+            # checking if files are suitable
             if return_code != 0:
                 return return_code
+            # setting cipher
             f = Fernet(key)
+            # getting data from file
             with open(path, "rb") as file:
                 file.read(3)
                 file.read(14)
                 ciphered_data = file.read()
+            # decrypting and writing to file
             decrypted_data = f.decrypt(ciphered_data)
             with open(path, 'wb') as file:
                 file.write(decrypted_data)
             return 0
+        # directory encryption
         if os.path.isdir(path) is True:
+            # looping through files
             for root, dirs, files in os.walk(path):
                 for file in files:
                     file_path = os.path.join(root, file)
+                    # checking if file is suitable
                     return_code = self.check_path_validity(file_path, decrypting=True, type_="FER")
                     if return_code != 0:
                         return return_code
+                    # setting cipher and getting data from file
                     f = Fernet(key)
                     with open(file_path, "rb") as file:
                         file.read(3)
                         file.read(14)
                         ciphered_data = file.read()
+                    # decrypting data and writing to file
                     decrypted_data = f.decrypt(ciphered_data)
                     with open(file_path, 'wb') as file:
                         file.write(decrypted_data)
             return 0
     
 
-    def FerNet_encrypt_string(self, key = str or None, bstring=bytes):
+    def FerNet_encrypt_string(self, key=None, bstring=bytes):
+        """
+        Method for encrypting a string with the `cryptography` module's `Fernet` class.
+
+        will return a tuple of `(encrypted_text, key)` if key is not provided.
+        
+         will return `encrypted_text` as a `bytes` object if the key is provided.
+        """
         if key is None:
             key = self.generate_FerNet_key()
-        f = Fernet(key)
-        encrypted_text = f.encrypt(bstring)
-        return encrypted_text
+            f = Fernet(key)
+            encrypted_text = f.encrypt(bstring)
+            return encrypted_text, key
+        else:
+            f = Fernet(key)
+            encrypted_text = f.encrypt(bstring)
+            return encrypted_text
 
 
     def FerNet_decrypt_string(self, key=bytes, encrypted_string=bytes):
+        """
+        Method for decryting a string with the `cryptography` module's `Fernet` class.
+
+        Will raise a `NoKeyError` if a key for the cipher was not provided.
+
+        Will return the decrypted text as a `bytes` object.
+        """
         if key is None:
-            return 1
+            raise NoKeyError("No key was provided for the cipher.")
         f = Fernet(key)
         decrypted_text = f.encrypt(encrypted_string)
         return decrypted_text
+
+
+class Cipher_Constructor:
+
+    def __init__(self) -> None:
+        pass
+
+
+    def intialize_AES_cipher(mode=str, key=None, iv=None, nonce=None) -> object:
+        # this template will be used for all the modes
+        if mode == "CFB":
+            # guard clauses
+            if iv is None:
+                raise InvalidCipherArgument("Intialization vector was not provided.")
+            elif key is None:
+                raise NoKeyError("Key was not provided.")
+            # generating cipher and returning
+            cipher = AES.new(key=key, mode=AES.MODE_CFB, iv=iv)
+            return cipher
+        elif mode == "CBC":
+            if iv is None:
+                raise InvalidCipherArgument("Intialization vector was not provided.")
+            elif key is None:
+                raise NoKeyError("Key was not provided.")
+            cipher = AES.new(key=key, mode=AES.MODE_CBC, iv=iv)
+            return cipher
+        elif mode == "CTR":
+            if nonce is None:
+                raise InvalidCipherArgument("Nonce was not provided.")
+            elif key is None:
+                raise NoKeyError("Key was not provided.")
+            cipher = AES.new(key=key, mode=AES.MODE_CTR, nonce=nonce)
+            return cipher
+        elif mode == "ECB":
+            if key is None:
+                raise NoKeyError("Key was not provided.")
+            cipher = AES.new(key=key, mode=AES.MODE_ECB)
+            return cipher
+        elif mode == "OFB":
+            if iv is None:
+                raise InvalidCipherArgument("Intialization vector was not provided.")
+            elif key is None:
+                raise NoKeyError("Key was not provided.")
+            cipher = AES.new(key=key, mode=AES.MODE_OFB, iv=iv)
+            return cipher
+        elif mode == "OPENPGP":
+            if iv is None:
+                raise InvalidCipherArgument("Intialization vector was not provided.")
+            elif key is None:
+                raise NoKeyError("Key was not provided.")
+            cipher = AES.new(key=key, mode=AES.MODE_OPENPGP, iv=iv)
+            return cipher
+        elif mode == "CCM":
+            if nonce is None:
+                raise InvalidCipherArgument("Nonce was not provided.")
+            elif key is None:
+                raise NoKeyError("Key was not provided.")
+            cipher = AES.new(key=key, mode=AES.MODE_CCM, nonce=nonce)
+            return cipher
+        elif mode == "EAX":
+            if nonce is None:
+                raise InvalidCipherArgument("Nonce was not provided.")
+            elif key is None:
+                raise NoKeyError("Key was not provided.")
+            cipher = AES.new(key=key, mode=AES.MODE_EAX, nonce=nonce)
+            return cipher
+        elif mode == "GCM":
+            if nonce is None:
+                raise InvalidCipherArgument("Nonce was not provided.")
+            elif key is None:
+                raise NoKeyError("Key was not provided.")
+            cipher = AES.new(key=key, mode=AES.MODE_GCM, nonce=nonce)
+            return cipher
+        elif mode == "OCB":
+            if nonce is None:
+                raise InvalidCipherArgument("Nonce was not provided.")
+            elif key is None:
+                raise NoKeyError("Key was not provided.")
+            cipher = AES.new(key=key, mode=AES.MODE_OCB, nonce=nonce)
+            return cipher
+
+    
+    def intialize_FerNet_cipher(key=bytes):
+        """
+        Returns a `Fernet` cipher object given the key.
+        """
+        if key is None:
+            raise NoKeyError("Key was not provided.")
+        fernet_cipher = Fernet(key=key)
+        return fernet_cipher
\ No newline at end of file
